#| WM/theme.jl -- WindowMaker-like theme with dockapp support

   Copyright (C) 2009 Alexey I. Froloff <raorn@altlinux.org>

   This file is not part of sawfish.

   This module is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published
   by the Free Software Foundation; either version 2, or (at your
   option) any later version.

   This module is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this module.  If not, write to the Free Software
   Foundation, 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
|#

(defgroup WM "WM Theme" :group appearance)

(defcustom WM:title-font default-font
        "Title string font: \\w"
        :group (appearance WM)
        :type font
        :after-set (lambda () (rebuild-frames-with-style 'WM)))

(define (get-title-font) WM:title-font)

(let*
  ((tile (make-image "tile.png"))
   (iconify-img (make-image "iconify.png"))
   (iconify-clicked-img (make-image "iconify-clicked.png"))
   (close-img (make-image "close.png"))
   (close-clicked-img (make-image "close-clicked.png"))
   (col-black (get-color "#000000"))
   (col-darkgray (get-color "#555555"))
   (col-gray (get-color "#909090"))
   (col-lightgray (get-color "#a8a8a8"))
   (col-white (get-color "#ffffff"))

   (resizable-p (lambda (w)
		  (let* ((hints (window-size-hints w))
			 (min-height (cdr (assq 'min-height hints)))
			 (max-height (cdr (assq 'max-height hints)))
			 (min-width (cdr (assq 'min-width hints)))
			 (max-width (cdr (assq 'max-width hints)))) 
		    (and (= min-height max-height)
			 (= min-width max-width)))))

   (tile-offset-x (lambda (w)
		    (- (/ (car (window-dimensions w)) 2) 32)))
   (tile-offset-y (lambda (w)
		    (- (/ (cdr (window-dimensions w)) 2) 32)))

   (appicon `(((background . ,tile)
	       (left-edge . ,tile-offset-x)
	       (top-edge . ,tile-offset-y)
	       (right-edge . ,tile-offset-x)
	       (bottom-edge . ,tile-offset-y)
	       (below-client . t)
	       (class . appicon))))

   (border-top `(((background . ,col-black)
		  (top-edge . -1)
		  (left-edge . -1)
		  (right-edge . -1)
		  (height . 1))))

   (border `(((background . ,col-black)
	      (bottom-edge . -1)
	      (left-edge . -1)
	      (right-edge . -1)
	      (height . 1))
	     ((background . ,col-black)
	      (top-edge . -1)
	      (bottom-edge . -1)
	      (left-edge . -1)
	      (width . 1))
	     ((background . ,col-black)
	      (top-edge . -1)
	      (bottom-edge . -1)
	      (right-edge . -1)
	      (width . 1))))

   (resizebar `(((background . ,col-black)
		 (bottom-edge . -9)
		 (left-edge . -1)
		 (right-edge . -1)
		 (height . 8))
		((background . ,col-darkgray)
		 (bottom-edge . -1)
		 (left-edge . 0)
		 (right-edge . 0)
		 (height . 1))
		((background . ,col-white)
		 (bottom-edge . -2)
		 (left-edge . 0)
		 (right-edge . 0)
		 (height . 1))
		((background . ,col-lightgray)
		 (bottom-edge . -8)
		 (left-edge . 0)
		 (width . 28)
		 (height . 6)
		 (class . bottom-left-corner))
		((background . ,col-lightgray)
		 (bottom-edge . -8)
		 (right-edge . 0)
		 (width . 28)
		 (height . 6)
		 (class . bottom-right-corner))
		((background . ,col-darkgray)
		 (bottom-edge . -8)
		 (left-edge . 28)
		 (width . 1)
		 (height . 6))
		((background . ,col-white)
		 (bottom-edge . -8)
		 (right-edge . 28)
		 (width . 1)
		 (height . 6))
		((background . ,col-white)
		 (bottom-edge . -8)
		 (left-edge . 29)
		 (width . 1)
		 (height . 6))
		((background . ,col-darkgray)
		 (bottom-edge . -8)
		 (right-edge . 29)
		 (width . 1)
		 (height . 6))
		((background . ,col-lightgray)
		 (bottom-edge . -8)
		 (left-edge . 30)
		 (right-edge . 30)
		 (height . 6)
		 (class . bottom-border))))

   (titlebar `(((background . ,col-black)
		(top-edge . -25)
		(left-edge . -1)
		(right-edge . -1)
		(height . 24)
		(class . title))
	       ((background . (,col-white ,col-lightgray))
		(top-edge . -24)
		(left-edge . 0)
		(right-edge . 0)
		(height . 23)
		(class . title))
	       ((background . ,col-darkgray)
		(top-edge . -23)
		(left-edge . 1)
		(right-edge . 0)
		(height . 22)
		(class . title))
	       ((background . (,col-lightgray ,col-black))
		(foreground . (,col-black ,col-white))
		(font . ,get-title-font)
		(text . ,window-name)
		(x-justify . center)
		(y-justify . center)
		(top-edge . -23)
		(left-edge . 1)
		(right-edge . 1)
		(height . 21)
		(class . title))))

   (iconify-button `(((background . (,iconify-img ,iconify-img ,iconify-img ,iconify-clicked-img))
		      (top-edge . -21)
		      (left-edge . 3)
		      (width . 17)
		      (height . 17)
		      (class . iconify-button))))

   (close-button `(((background . (,close-img ,close-img ,close-img ,close-clicked-img))
		    (top-edge . -21)
		    (right-edge . 4)
		    (width . 17)
		    (height . 17)
		    (class . close-button))))

   (dialog-title `(,@titlebar ,@close-button))
   (full-title `(,@titlebar ,@iconify-button ,@close-button))
   (normal-noresize `(,@full-title ,@border-top ,@border))
   (normal `(,@normal-noresize ,@resizebar))
   (shaded `(,@full-title ,@border-top))
   (shaped `(,@full-title ,@border-top))
   (transient-noresize `(,@dialog-title ,@border-top ,@border))
   (transient `(,@dialog-title ,@transient-noresize ,@resizebar))
   )

  (add-frame-style 'WM
		   (lambda (w type)
		     (case type
		       ((dockapp) appicon)
		       ((shaped) shaped)
		       ((shaded) shaded)
		       ((transient) (if (resizable-p w)
					transient-noresize
				      transient))
		       ((shaped-transient) )
		       ((default) (if (resizable-p w)
				      normal-noresize
				    normal))))))
